@@ -1,15 +1,17 @@
 import mock
-import unittest
+import sys
+
+import six
 
 from ...errors import OkonomiyakiError
 
 from .. import EPDPlatform
 from ..epd_platform import (
-    _guess_epd_platform, EPD_PLATFORM_SHORT_NAMES, applies
+    _guess_epd_platform, EPD_PLATFORM_SHORT_NAMES, X86, X86_64, applies
 )
 from ..legacy import _SUBDIR
-from ..platform import DARWIN, LINUX, MAC_OS_X, RHEL, WINDOWS
-from .._arch import Arch, X86, X86_64
+from ..platform import OSKind, FamilyKind, NameKind
+from .._arch import Arch
 
 from .common import (
     mock_architecture_32bit, mock_architecture_64bit, mock_centos_5_8,
@@ -17,8 +19,20 @@ from .common import (
     mock_solaris, mock_ubuntu_raring, mock_windows, mock_x86, mock_x86_64
 )
 
+if sys.version_info < (2, 7):
+    import unittest2 as unittest
+else:
+    import unittest
+
 
 class TestEPDPlatform(unittest.TestCase):
+    def setUp(self):
+        self.platform_strings = tuple(
+            platform + "-{0}".format(arch)
+            for arch in ("x86", "x86_64")
+            for platform in ("osx", "win", "rh5")
+        )
+
     def test_short_names_consistency(self):
         legacy_entries = sorted([entry[0] for entry in _SUBDIR])
 
@@ -27,7 +41,25 @@ class TestEPDPlatform(unittest.TestCase):
     def test_epd_platform_from_string(self):
         """Ensure every epd short platform is understood by EPDPlatform."""
         for epd_platform_string in EPD_PLATFORM_SHORT_NAMES:
-            EPDPlatform.from_epd_string(epd_platform_string)
+            EPDPlatform.from_string(epd_platform_string)
+
+    def test_pep425_is_unicode(self):
+        # When/Then
+        for platform_string in self.platform_strings:
+            platform = EPDPlatform.from_string(platform_string)
+            self.assertIsInstance(platform.pep425_tag, six.text_type)
+
+    def test_platform_name(self):
+        # When/Then
+        for platform_string in self.platform_strings:
+            platform = EPDPlatform.from_string(platform_string)
+            self.assertIsInstance(platform.platform_name, six.text_type)
+
+    def test_str_is_unicode(self):
+        # When/Then
+        for platform_string in self.platform_strings:
+            platform = EPDPlatform.from_string(platform_string)
+            self.assertIsInstance(six.text_type(platform), six.text_type)
 
     def test_epd_platform_from_string_new_names_underscore(self):
         # Given
@@ -35,7 +67,7 @@ class TestEPDPlatform(unittest.TestCase):
 
         # When
         epd_platforms = tuple(
-            EPDPlatform.from_epd_string("rh5_" + arch)
+            EPDPlatform.from_string("rh5_" + arch)
             for arch in archs
         )
 
@@ -48,7 +80,7 @@ class TestEPDPlatform(unittest.TestCase):
 
         # When
         epd_platforms = tuple(
-            EPDPlatform.from_epd_string("rh5_" + arch)
+            EPDPlatform.from_string("rh5_" + arch)
             for arch in archs
         )
 
@@ -60,19 +92,19 @@ class TestEPDPlatform(unittest.TestCase):
         s = "win_x86_64"
 
         # When
-        epd_platform = EPDPlatform.from_epd_string(s)
+        epd_platform = EPDPlatform.from_string(s)
 
         # Then
-        self.assertEqual(epd_platform, EPDPlatform.from_epd_string("win-64"))
+        self.assertEqual(epd_platform, EPDPlatform.from_string("win-64"))
 
         # Given
         s = "osx_x86_64"
 
         # When
-        epd_platform = EPDPlatform.from_epd_string(s)
+        epd_platform = EPDPlatform.from_string(s)
 
         # Then
-        self.assertEqual(epd_platform, EPDPlatform.from_epd_string("osx-64"))
+        self.assertEqual(epd_platform, EPDPlatform.from_string("osx-64"))
 
     def test_epd_platform_from_string_new_names(self):
         """Ensure every epd short platform is understood by EPDPlatform."""
@@ -81,7 +113,7 @@ class TestEPDPlatform(unittest.TestCase):
 
         # When
         epd_platforms = tuple(
-            EPDPlatform.from_epd_string("rh5-" + arch)
+            EPDPlatform.from_string("rh5-" + arch)
             for arch in archs
         )
 
@@ -94,7 +126,7 @@ class TestEPDPlatform(unittest.TestCase):
 
         # When
         epd_platforms = tuple(
-            EPDPlatform.from_epd_string("rh5-" + arch)
+            EPDPlatform.from_string("rh5-" + arch)
             for arch in archs
         )
 
@@ -139,7 +171,7 @@ class TestEPDPlatform(unittest.TestCase):
     def test_epd_platform_from_string_new_arch(self):
         def old_to_new_name(epd_platform_string):
             left, right = epd_platform_string.split("-")
-            return "{}-{}".format(left, {"32": X86, "64": X86_64}[right])
+            return "{0}-{1}".format(left, {"32": X86, "64": X86_64}[right])
 
         # Given
         name_to_platform = {}
@@ -147,7 +179,7 @@ class TestEPDPlatform(unittest.TestCase):
         # When
         for old_name in EPD_PLATFORM_SHORT_NAMES:
             new_name = old_to_new_name(old_name)
-            name_to_platform[old_name] = EPDPlatform.from_epd_string(new_name)
+            name_to_platform[old_name] = EPDPlatform.from_string(new_name)
 
         # Then
         for old_name in name_to_platform:
@@ -163,34 +195,69 @@ class TestEPDPlatform(unittest.TestCase):
 
     def test_str(self):
         # Given
-        epd_platform = EPDPlatform.from_epd_string("rh5-64")
+        epd_platform = EPDPlatform.from_string("rh5-64")
 
         # When/Then
         self.assertEqual(str(epd_platform), "rh5_x86_64")
 
         # Given
-        epd_platform = EPDPlatform.from_epd_string("osx-32")
+        epd_platform = EPDPlatform.from_string("osx-32")
 
         # When/Then
         self.assertEqual(str(epd_platform), "osx_x86")
+        self.assertEqual(six.text_type(epd_platform), u"osx_x86")
+        self.assertIsInstance(six.text_type(epd_platform), six.text_type)
 
         # Given
         s = "osx_x86"
 
         # When
-        epd_platform = EPDPlatform.from_epd_string(s)
+        epd_platform = EPDPlatform.from_string(s)
 
         # Then
         self.assertEqual(str(epd_platform), s)
 
+    def test_pep425_tag(self):
+        # Given
+        epd_string_to_pep425 = (
+            ("rh5_x86", "linux_i686"),
+            ("rh5_x86_64", "linux_x86_64"),
+            ("osx_x86", "macosx_10_6_i386"),
+            ("osx_x86_64", "macosx_10_6_x86_64"),
+            ("win_x86", "win32"),
+            ("win_x86_64", "win_amd64"),
+        )
+
+        # When/Then
+        for epd_string, platform_tag in epd_string_to_pep425:
+            self.assertEqual(
+                EPDPlatform.from_string(epd_string).pep425_tag,
+                platform_tag,
+            )
+
 
 class TestEPDPlatformApplies(unittest.TestCase):
+    @mock_centos_5_8
+    def test_no_arch(self):
+        with mock_x86:
+            self.assertTrue(applies("rh5", "current"))
+            self.assertFalse(applies("!rh5", "current"))
+
+        platform = EPDPlatform.from_string("rh5-x86_64")
+        self.assertTrue(applies("rh5", platform))
+        self.assertFalse(applies("!rh5", platform))
+        self.assertFalse(applies("rh5-32", platform))
+
     @mock_centos_5_8
     def test_all(self):
         with mock_x86:
             self.assertTrue(applies("all", "current"))
             self.assertFalse(applies("!all", "current"))
 
+        platform = EPDPlatform.from_string("rh5-x86_64")
+        self.assertTrue(applies("all", platform))
+        self.assertFalse(applies("!all", platform))
+
     @mock_centos_5_8
     def test_current_linux(self):
         with mock_x86:
@@ -344,21 +411,21 @@ class TestGuessEPDPlatform(unittest.TestCase):
         # Given
         examples = (
             (("linux2", None, "x86"),
-             EPDPlatform.from_epd_string("rh5-32"),),
+             EPDPlatform.from_string("rh5-32"),),
             (("linux2", "RedHat_3", "x86"),
-             EPDPlatform.from_epd_string("rh3-32"),),
+             EPDPlatform.from_string("rh3-32"),),
             (("linux2", "RedHat_5", "x86"),
-             EPDPlatform.from_epd_string("rh5-32"),),
+             EPDPlatform.from_string("rh5-32"),),
             (("linux2", "RedHat_5", "amd64"),
-             EPDPlatform.from_epd_string("rh5-64"),),
+             EPDPlatform.from_string("rh5-64"),),
             (("darwin", None, "x86"),
-             EPDPlatform.from_epd_string("osx-32"),),
+             EPDPlatform.from_string("osx-32"),),
             (("darwin", None, "amd64"),
-             EPDPlatform.from_epd_string("osx-64"),),
+             EPDPlatform.from_string("osx-64"),),
             (("win32", None, "x86"),
-             EPDPlatform.from_epd_string("win-32"),),
+             EPDPlatform.from_string("win-32"),),
             (("win32", None, "amd64"),
-             EPDPlatform.from_epd_string("win-64"),),
+             EPDPlatform.from_string("win-64"),),
         )
 
         # When/Then
@@ -371,43 +438,43 @@ class TestGuessEPDPlatform(unittest.TestCase):
         epd_platform_string = "rh5-32"
 
         # When
-        epd_platform = EPDPlatform.from_epd_string(epd_platform_string)
+        epd_platform = EPDPlatform.from_string(epd_platform_string)
 
         # Then
         platform = epd_platform.platform
-        self.assertEqual(platform.os, LINUX)
-        self.assertEqual(platform.family, RHEL)
-        self.assertEqual(platform.name, RHEL)
-        self.assertEqual(platform.arch, Arch.from_name(X86))
-        self.assertEqual(platform.machine, Arch.from_name(X86))
+        self.assertEqual(platform.os_kind, OSKind.linux)
+        self.assertEqual(platform.family_kind, FamilyKind.rhel)
+        self.assertEqual(platform.name_kind, NameKind.rhel)
+        self.assertEqual(platform.arch, X86)
+        self.assertEqual(platform.machine, X86)
 
         # Given
         epd_platform_string = "win-32"
 
         # When
-        epd_platform = EPDPlatform.from_epd_string(epd_platform_string)
+        epd_platform = EPDPlatform.from_string(epd_platform_string)
 
         # Then
         platform = epd_platform.platform
-        self.assertEqual(platform.os, WINDOWS)
-        self.assertEqual(platform.family, WINDOWS)
-        self.assertEqual(platform.name, WINDOWS)
-        self.assertEqual(platform.arch, Arch.from_name(X86))
-        self.assertEqual(platform.machine, Arch.from_name(X86))
+        self.assertEqual(platform.os_kind, OSKind.windows)
+        self.assertEqual(platform.family_kind, FamilyKind.windows)
+        self.assertEqual(platform.name_kind, NameKind.windows)
+        self.assertEqual(platform.arch, X86)
+        self.assertEqual(platform.machine, X86)
 
         # Given
         epd_platform_string = "osx-64"
 
         # When
-        epd_platform = EPDPlatform.from_epd_string(epd_platform_string)
+        epd_platform = EPDPlatform.from_string(epd_platform_string)
         platform = epd_platform.platform
 
         # Then
-        self.assertEqual(platform.os, DARWIN)
-        self.assertEqual(platform.family, MAC_OS_X)
-        self.assertEqual(platform.name, MAC_OS_X)
-        self.assertEqual(platform.arch, Arch.from_name(X86_64))
-        self.assertEqual(platform.machine, Arch.from_name(X86_64))
+        self.assertEqual(platform.os_kind, OSKind.darwin)
+        self.assertEqual(platform.family_kind, FamilyKind.mac_os_x)
+        self.assertEqual(platform.name_kind, NameKind.mac_os_x)
+        self.assertEqual(platform.arch, X86_64)
+        self.assertEqual(platform.machine, X86_64)
 
     def test_from_epd_platform_string_invalid(self):
         # Given
@@ -416,7 +483,7 @@ class TestGuessEPDPlatform(unittest.TestCase):
 
         # When/Then
         with self.assertRaises(OkonomiyakiError):
-            EPDPlatform.from_epd_string(epd_platform_string)
+            EPDPlatform.from_string(epd_platform_string)
 
         # Given
         # Invalid bitwidth
@@ -424,7 +491,7 @@ class TestGuessEPDPlatform(unittest.TestCase):
 
         # When/Then
         with self.assertRaises(OkonomiyakiError):
-            EPDPlatform.from_epd_string(epd_platform_string)
+            EPDPlatform.from_string(epd_platform_string)
 
         # Given
         # Invalid platform basename
@@ -432,4 +499,31 @@ class TestGuessEPDPlatform(unittest.TestCase):
 
         # When/Then
         with self.assertRaises(OkonomiyakiError):
-            EPDPlatform.from_epd_string(epd_platform_string)
+            EPDPlatform.from_string(epd_platform_string)
+
+    def test_from_platform_tag(self):
+        # Given
+        inputs_outputs = (
+            ("linux_i686", "rh5-32"),
+            ("linux_i386", "rh5-32"),
+            ("linux_x86_64", "rh5-64"),
+            ("win32", "win-32"),
+            ("win_amd64", "win-64"),
+            ("macosx_10_6_x86_64", "osx-64"),
+            ("macosx_10_6_i386", "osx-32"),
+        )
+
+        # When/Then
+        for platform_tag, epd_string in inputs_outputs:
+            platform = EPDPlatform._from_platform_tag(platform_tag)
+            self.assertEqual(platform.short, epd_string)
+
+        # When/Then
+        with self.assertRaises(NotImplementedError):
+            EPDPlatform._from_platform_tag("openbsd_i386")
+
+        with self.assertRaises(ValueError):
+            EPDPlatform._from_platform_tag("any")
+
+        with self.assertRaises(ValueError):
+            EPDPlatform._from_platform_tag(None)
